@@ -43,13 +43,12 @@
 
 #include <gdk/gdkkeysyms.h>
 
-#include <glade/glade-xml.h>
-
 #include "seahorse-libdialogs.h"
 #include "seahorse-widget.h"
 #include "seahorse-util.h"
 #include "seahorse-passphrase.h"
-#include "seahorse-secure-entry.h"
+
+#include "common/seahorse-secure-buffer.h"
 
 #define HIG_SMALL      6        /* gnome hig small space in pixels */
 #define HIG_LARGE     12        /* gnome hig large space in pixels */
@@ -90,31 +89,37 @@ key_press (GtkWidget *widget, GdkEventKey *event, gpointer data)
     return FALSE;
 }
 
-static void
+static gboolean
 grab_keyboard (GtkWidget *win, GdkEvent *event, gpointer data)
 {
 #ifndef _DEBUG
-    if (gdk_keyboard_grab (win->window, FALSE, gdk_event_get_time (event)))
-        g_critical ("could not grab keyboard");
+	if (!g_object_get_data (G_OBJECT (win), "keyboard-grabbed"))
+		if (gdk_keyboard_grab (win->window, FALSE, gdk_event_get_time (event)))
+			g_message ("could not grab keyboard");
+	g_object_set_data (G_OBJECT (win), "keyboard-grabbed", GINT_TO_POINTER (TRUE));
 #endif
+	return FALSE;
 }
 
 /* ungrab_keyboard - remove grab */
-static void
+static gboolean
 ungrab_keyboard (GtkWidget *win, GdkEvent *event, gpointer data)
 {
 #ifndef _DEBUG
-    gdk_keyboard_ungrab (gdk_event_get_time (event));
+	if (g_object_get_data (G_OBJECT (win), "keyboard-grabbed"))
+		gdk_keyboard_ungrab (gdk_event_get_time (event));
+	g_object_set_data (G_OBJECT (win), "keyboard-grabbed", NULL);
 #endif
+	return FALSE;
 }
 
 /* When enter is pressed in the confirm entry, move */
 static void
 confirm_callback (GtkWidget *widget, GtkDialog *dialog)
 {
-    GtkWidget *entry = GTK_WIDGET (g_object_get_data (G_OBJECT (dialog), "secure-entry"));
-    g_assert (SEAHORSE_IS_SECURE_ENTRY (entry));
-    gtk_widget_grab_focus (entry);
+	GtkWidget *entry = GTK_WIDGET (g_object_get_data (G_OBJECT (dialog), "secure-entry"));
+	g_assert (GTK_IS_ENTRY (entry));
+	gtk_widget_grab_focus (entry);
 }
 
 /* When enter is pressed in the entry, we simulate an ok */
@@ -127,14 +132,30 @@ enter_callback (GtkWidget *widget, GtkDialog *dialog)
 static void
 entry_changed (GtkEditable *editable, GtkDialog *dialog)
 {
-    SeahorseSecureEntry *entry, *confirm;
-    
-    entry = SEAHORSE_SECURE_ENTRY (g_object_get_data (G_OBJECT (dialog), "secure-entry"));
-    confirm = SEAHORSE_SECURE_ENTRY (g_object_get_data (G_OBJECT (dialog), "confirm-entry"));
-    
-    gtk_dialog_set_response_sensitive (dialog, GTK_RESPONSE_ACCEPT, 
-                                       strcmp (seahorse_secure_entry_get_text (entry), 
-                                               seahorse_secure_entry_get_text (confirm)) == 0);
+	GtkEntry *entry, *confirm;
+
+	entry = GTK_ENTRY (g_object_get_data (G_OBJECT (dialog), "secure-entry"));
+	confirm = GTK_ENTRY (g_object_get_data (G_OBJECT (dialog), "confirm-entry"));
+
+	gtk_dialog_set_response_sensitive (dialog, GTK_RESPONSE_ACCEPT,
+	                                   strcmp (gtk_entry_get_text (entry),
+	                                           gtk_entry_get_text (confirm)) == 0);
+}
+
+static gboolean
+window_state_changed (GtkWidget *win, GdkEventWindowState *event, gpointer data)
+{
+	GdkWindowState state = gdk_window_get_state (win->window);
+	
+	if (state & GDK_WINDOW_STATE_WITHDRAWN ||
+	    state & GDK_WINDOW_STATE_ICONIFIED ||
+	    state & GDK_WINDOW_STATE_FULLSCREEN ||
+	    state & GDK_WINDOW_STATE_MAXIMIZED)
+	    	ungrab_keyboard (win, (GdkEvent*)event, data);
+	else
+		grab_keyboard (win, (GdkEvent*)event, data);
+		
+	return FALSE;
 }
 
 static void
@@ -160,7 +181,8 @@ seahorse_passphrase_prompt_show (const gchar *title, const gchar *description,
                                  const gchar *prompt, const gchar *check,
                                  gboolean confirm)
 {
-    SeahorseSecureEntry *entry;
+    GtkEntryBuffer *buffer;
+    GtkEntry *entry;
     GtkDialog *dialog;
     GtkWidget *w;
     GtkWidget *box;
@@ -181,11 +203,10 @@ seahorse_passphrase_prompt_show (const gchar *title, const gchar *description,
 
     dialog = GTK_DIALOG (w);
 
-    g_signal_connect (G_OBJECT (dialog), "size-request",
-                      G_CALLBACK (constrain_size), NULL);
-
+    g_signal_connect (G_OBJECT (dialog), "size-request", G_CALLBACK (constrain_size), NULL);
     g_signal_connect (G_OBJECT (dialog), "map-event", G_CALLBACK (grab_keyboard), NULL);
     g_signal_connect (G_OBJECT (dialog), "unmap-event", G_CALLBACK (ungrab_keyboard), NULL);
+    g_signal_connect (G_OBJECT (dialog), "window-state-event", G_CALLBACK (window_state_changed), NULL); 
 
     wvbox = gtk_vbox_new (FALSE, HIG_LARGE * 2);
     gtk_container_add (GTK_CONTAINER (dialog->vbox), wvbox);
@@ -227,7 +248,10 @@ seahorse_passphrase_prompt_show (const gchar *title, const gchar *description,
         g_free (msg);
         gtk_table_attach (table, w, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
 
-        entry = SEAHORSE_SECURE_ENTRY (seahorse_secure_entry_new ());
+        buffer = seahorse_secure_buffer_new ();
+        entry = GTK_ENTRY (gtk_entry_new_with_buffer (buffer));
+        g_object_unref (buffer);
+        gtk_entry_set_visibility (entry, FALSE);
         gtk_widget_set_size_request (GTK_WIDGET (entry), 200, -1);
         g_object_set_data (G_OBJECT (dialog), "confirm-entry", entry);
         g_signal_connect (G_OBJECT (entry), "activate", G_CALLBACK (confirm_callback), dialog);
@@ -242,9 +266,12 @@ seahorse_passphrase_prompt_show (const gchar *title, const gchar *description,
     w = gtk_label_new (msg);
     g_free (msg);
     gtk_table_attach (table, w, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
-    
-    entry = SEAHORSE_SECURE_ENTRY (seahorse_secure_entry_new ());
+
+    buffer = seahorse_secure_buffer_new ();
+    entry = GTK_ENTRY (gtk_entry_new_with_buffer (buffer));
+    g_object_unref (buffer);
     gtk_widget_set_size_request (GTK_WIDGET (entry), 200, -1);
+    gtk_entry_set_visibility (entry, FALSE);
     g_object_set_data (G_OBJECT (dialog), "secure-entry", entry);
     g_signal_connect (G_OBJECT (entry), "activate", G_CALLBACK (enter_callback), dialog);
     gtk_table_attach_defaults (table, GTK_WIDGET (entry), 1, 2, 1, 2);
@@ -276,7 +303,9 @@ seahorse_passphrase_prompt_show (const gchar *title, const gchar *description,
     g_signal_connect (dialog, "key_press_event", G_CALLBACK (key_press), NULL);
 
     gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
-    gtk_window_set_keep_above(GTK_WINDOW (dialog), TRUE);
+    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);    
+    gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_NORMAL);
+    gtk_window_set_keep_above (GTK_WINDOW (dialog), TRUE);
     gtk_widget_show_all (GTK_WIDGET (dialog));
     gdk_window_focus (GTK_WIDGET (dialog)->window, GDK_CURRENT_TIME);
 
@@ -289,10 +318,8 @@ seahorse_passphrase_prompt_show (const gchar *title, const gchar *description,
 const gchar*
 seahorse_passphrase_prompt_get (GtkDialog *dialog)
 {
-    SeahorseSecureEntry *entry;
-
-    entry = SEAHORSE_SECURE_ENTRY (g_object_get_data (G_OBJECT (dialog), "secure-entry"));
-    return seahorse_secure_entry_get_text (entry);
+    GtkEntry *entry = GTK_ENTRY (g_object_get_data (G_OBJECT (dialog), "secure-entry"));
+    return gtk_entry_get_text (entry);
 }
 
 gboolean
